#
# Copyright (c) 2013-2023 NIBIO.
#
# This file is part of VIPSWeb 
# (see https://gitlab.nibio.no/VIPS/VIPSWeb).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
#


from django.urls import re_path, path, include
#from django.views.generic import TemplateView
from django.views.static import serve
from django.views.i18n import JavaScriptCatalog
from django.conf import settings
from VIPSWeb import views

# Uncomment the next two lines to enable the admin:
from django.contrib import admin
admin.autodiscover()

# Enabling translation in JavaScript files
# See https://docs.djangoproject.com/en/1.5/topics/i18n/translation/#internationalization-in-javascript-code
js_info_dict = {
    'packages': ['forecasts','vips_messages','VIPSWeb','roughage', 'spatial','cydiapomonella'],
    'domain': 'djangojs',
}

#print "settings.MAINTENANCE_MODE=%s" % settings.MAINTENANCE_MODE

if settings.MAINTENANCE_MODE is True:
    urlpatterns = [
                           re_path(r'^$', views.maintenance, name='maintenance')
    ]
else:
    urlpatterns = [
        # Examples:
        # url(r'^$', 'VIPSWeb.views.home', name='home'),
        # url(r'^VIPSWeb/', include('VIPSWeb.foo.urls')),
    
        # Uncomment the admin/doc line below to enable admin documentation:
        # url(r'^admin/doc/', include('django.contrib.admindocs.urls')),

        re_path(r'^spatial/', include('spatial.urls')),
	    re_path(r'^cydiapomonella/', include('cydiapomonella.urls')),
        re_path(r'^forecasts/', include('forecasts.urls', namespace = "forecasts")),
        re_path(r'^messages/', include('vips_messages.urls', namespace = "vips_messages")),
        re_path(r'^organisms/', include('organisms.urls', namespace = "organisms")),
        re_path(r'^observations/', include('observations.urls', namespace = "observations")),
        re_path(r'^information/', include('information.urls', namespace = "information")),
        re_path(r'^blotch/', include('cerealblotchmodels.urls', namespace = "cerealblotchmodels")),
        re_path(r'^ipmd/', include('ipmd.urls', namespace = "ipmd")),
        re_path(r'^calculators/', include('calculators.urls', namespace = "calculators")),
        re_path(r'^roughage/', include('roughage.urls', namespace = "roughage")),
        re_path(r'^security/', include('security.urls', namespace = "security")),
        re_path(r'^fusarium/', include('fusarium.urls', namespace = "fusarium")),
        re_path(r'^applefruitmoth/', include('applefruitmoth.urls', namespace = "applefruitmoth")),
        #re_path(r'^mock/', include('mock.urls', namespace = "mock")),
        # Uncomment the next line to enable the admin:
        re_path(r'^admin/', admin.site.urls),
        re_path(r'^tinymce/', include('tinymce.urls')),
        # Static serving of media files
        re_path(r'^media/(?P<path>.*)$', serve, {'document_root': settings.MEDIA_ROOT}),
        # Enabling translation in JavaScript files
        # See https://docs.djangoproject.com/en/1.6/topics/i18n/translation/#internationalization-in-javascript-code
        path('jsi18n/', JavaScriptCatalog.as_view(packages=js_info_dict['packages']), name='javascript-catalog'),
        #path('jsi18n/', JavaScriptCatalog.as_view(), name='javascript-catalog'),
        re_path(r'^i18n/', include('django.conf.urls.i18n')),
        path('settings.js', views.settings_js, name="views.settings_js"),
        re_path(r'^vipslogicproxy/(?P<path>.*)$', views.vipslogicproxy),
        # Default view is index
        re_path(r'^$', views.index, name='index'),
        #url(r'^$', TemplateView.as_view(template_name="index.html"))
    ]
