#
# Copyright (c) 2013-2023 NIBIO.
#
# This file is part of VIPSWeb 
# (see https://gitlab.nibio.no/VIPS/VIPSWeb).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
#
import datetime
from django.shortcuts import render
from django.http import HttpResponse,  HttpResponseBadRequest, HttpResponseNotAllowed  
from django.conf import settings
from django.utils import translation
import requests
from vips_messages.models import Advertisement
from organisms.models import CropCategory
from django.views.decorators.gzip import gzip_page
from django.views.decorators.cache import cache_page

def index(request):
    try:
        # Check if user is on stock Android browser, which performs poorly with OpenLayers
        user_is_stock_android = False
        user_agent = request.META.get('HTTP_USER_AGENT',"")
        
        if user_agent.find("Android") >= 0 and user_agent.find("Windows") < 0:
            chrome_index = user_agent.find("Chrome")
            if  chrome_index < 0:
                user_is_stock_android = True
            else:
                chrome_version = user_agent[chrome_index+7:chrome_index+9]
                if int(chrome_version) < 40:
                    user_is_stock_android = True
        
        user_uuid = request.GET.get("returnUUID",None)
        if user_uuid == None:
            user_uuid = request.session.get("user_uuid","")
        
        # Get front page categories. This is defined in local_settings.py
        #message_tags = MessageTag.get_message_tags(translation.get_language())
        # Get advertisements, if any
        #advertisements = Advertisement.objects.exclude(pub_date__gte=datetime.date.today()).exclude(exp_date__lte=datetime.date.today()).order_by('-pub_date')[:1]
        advertisements = Advertisement.objects.exclude(pub_date__gt=datetime.date.today()).exclude(exp_date__lt=datetime.date.today()).order_by('-pub_date')[:1]
        # Last 10 messages
        context = {
                   'user_uuid': user_uuid,
                  'advertisements': advertisements,
                  'crop_categories': CropCategory.get_crop_categories(translation.get_language()),
                  'user_is_stock_android':  user_is_stock_android,
                  }
        return render(request, 'index.html', context)
    except ValueError:
        return render(request, '503.html')

# Serving settings for JavaScript
def settings_js(request):
    user_agent = request.META['HTTP_USER_AGENT']
    user_is_ie = False
    if user_agent.find("MSIE") >= 0 or user_agent.find("Trident") >= 0:
            user_is_ie = True
    context = {
        'crop_categories': CropCategory.get_crop_categories_from_vipslogic_as_text(),
        'user_is_ie':  user_is_ie,
    }
    return render(request,'settings.js', context, content_type="application/javascript;charset=UTF-8")

## TODO: Is this being used???
def resourcebundle(request):
    return render(request, 'resourcebundle.js',content_type="application/javascript")

def maintenance(request):
    return render(request, '503.html')

@cache_page(60 * 10)
@gzip_page
def vipslogicproxy(request, path):
        
    """
    Proxy to use cross domain Ajax GET and POST requests
    request: Django request object
    """
    if request.method == 'GET':
        request = request.GET
        r = requests.get
    elif request.method == 'POST':
        request = request.POST
        r = requests.post
    else:
        return HttpResponseNotAllowed("Permitted methods are POST and GET")
    params = request.urlencode()

    try:
        url = "%s://%s/%s" % (settings.VIPSLOGIC_PROTOCOL, settings.VIPSLOGIC_SERVER_NAME,path)
        #print(url)
    except KeyError:
        return HttpResponseBadRequest("URL must be defined")
    url = url.encode("ascii")
    params_encoded = params.encode("ascii")
    response = r(url, params=params_encoded)
    if response.status_code == 200:
        content_type = response.headers.get('content-type')
        if 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' in content_type: 
            response_content = response.content
            content_disposition = response.headers.get('Content-Disposition', '')
            response = HttpResponse(response_content, content_type='application/vnd.openxmlformats-officedocument.spreadsheetml.sheet')
            if 'filename=' in content_disposition:
                filename = content_disposition.split('filename=')[1].strip().strip('"')
                response['Content-Disposition'] = f'attachment; filename="{filename}"'
            return response
        else:
            # Handle other content types normally
            return HttpResponse(response.text, status=response.status_code, content_type=content_type)
    else:
        return HttpResponse("[]", status=200, content_type="application/json")
