
class ApplefruitMoth 
{
	serverUri = settings.vipsCoremanagerServerName;
	runUri= this.serverUri + "/models/DAYDEGREES/run"; 
	weatherStationDataURL = "https://lmt.nibio.no/services/rest/vips/getdata/forecastfallback";
    gridDataURL = "https://weather.vips.nibio.no/rest/grid/openmeteo/"; 
    TIMEZONE_OSLO = "Europe/Oslo";
    DATE_FORMAT = "YYYY-MM-DD";
    WEATHER_DATASOURCE_TYPE_COORDINATES = "coordinates";
    WEATHER_DATASOURCE_TYPE_WEATHERSTATION = "weatherstation";
	
	basisTemp = "0";
	
    constructor() {
        this.weatherStationIds = [];
        this.weatherStations = [];
    }

    renderWeatherstations = async function () {
		console.info("Load weatherstations - start")
        try {
            const weatherStations = await window.lmtPoiMap();
            this.weatherStations = weatherStations;
            this.weatherStationIds = weatherStations.map(ws => ws.id);

            const select = document.getElementById('weatherStationId');

            // Add an option for each weather station
            this.weatherStations.forEach(station => {
                const option = document.createElement('option');
                option.value = station.id;
                option.textContent = station.name;
                select.appendChild(option);
            });
			console.info("Load weatherstations - finished")
        } catch (error) {
            console.error('Error:', error);
        }
    }

    getSelectedWeatherstation()
    {
        let wsSelect = document.getElementById("weatherStationId");
        return wsSelect.options[wsSelect.selectedIndex].value != "-1" ? wsSelect.options[wsSelect.selectedIndex].value : undefined;
    }
    
    getSelectedCoordinate()
    {
        let longitude = parseFloat(document.getElementById("longitude").value);
        let latitude = parseFloat(document.getElementById("latitude").value);
        return (Number.isNaN(longitude) || Number.isNaN(latitude) ? undefined : [longitude, latitude]);
    }

	getSelectedDates()
	{
		let startDate = document.getElementById("startdate").value;
		let endDate = document.getElementById("enddate").value;
		
		if(!startDate)
		{
			let date = new Date();
			date.setDate(date.getDate() - 7);
			date.setDate(date.getDate());
			startDate = date.toISOString().split('T')[0];
			
			document.getElementById("startdate").value = startDate;
		}
		
		if(!endDate){
			let date = new Date();
			date.setDate(date.getDate() + 9);
			endDate = date.toISOString().split('T')[0];
			
			document.getElementById("enddate").value = endDate;
		}
		
		return [startDate, endDate];
		
	}
	
	
    getSelectedWeatherdataSourceType()
    {
        return document.getElementById("coordinates").checked ? this.WEATHER_DATASOURCE_TYPE_COORDINATES : document.getElementById("weatherstation").checked ? this.WEATHER_DATASOURCE_TYPE_WEATHERSTATION : undefined;
    }
	
	/**
 	* Calls the dayDegreemodel in the VIPS forecasting system
 	* Displays data when results are returned
 	*/
	runModel () {
		    	    	
    	let selectedDates = this.getSelectedDates();
        let selectedWeatherStationId = this.getSelectedWeatherstation();
        let selectedCoordinate = this.getSelectedCoordinate();
        
        if(selectedWeatherStationId == undefined && selectedCoordinate == undefined)
        {
            return;
        }
        
        let stationName;
        
        let startDate = selectedDates[0];
        let endDate = selectedDates[1];


		
        try
        {
            let dailyDataResponse = undefined;
          
            if(selectedWeatherStationId != undefined && this.getSelectedWeatherdataSourceType() == this.WEATHER_DATASOURCE_TYPE_WEATHERSTATION)
            {
				stationName = this.weatherStations.find(ws => ws.id === Number(selectedWeatherStationId)).name;
                // Get the hourly data for the past X days
                dailyDataResponse = fetch(
                    this.weatherStationDataURL + "?weatherStationId=" + selectedWeatherStationId 
                    + "&elementMeasurementTypes[]=TM"
                    + "&timeZone="  + this.TIMEZONE_OSLO 
                    + "&startDate=" + startDate 
                    + "&endDate=" + endDate 
                    + "&logIntervalId=2"
                );
                
            }
            else if(this.getSelectedWeatherdataSourceType() == this.WEATHER_DATASOURCE_TYPE_COORDINATES && selectedCoordinate != undefined)
            {
				stationName = 'valgte koordinater';
                dailyDataResponse = fetch(
                    this.gridDataURL + "?longitude=" + selectedCoordinate[0] + "&latitude=" + selectedCoordinate[1]
                    + "&elementMeasurementTypes[]=TM"
                    + "&timeZone="  + document.getElementById("timezone").value
                    + "&startDate=" + startDate
                    + "&endDate=" + endDate  
                    + "&logIntervalId=2"
                );
            }
            else
            {
                console.error("Location not set. Aborting.")
                return;
            }
            dailyDataResponse
    			.then(response => {
        			if (!response.ok) {
            			throw new Error("Response status: " + response.status);
        			}
        			return response.json();
    		})
    		.then(dailyData => {
        		this.getDaydegrees(dailyData, startDate, endDate, stationName);
    		})
    		.catch(error => console.error('Error:', error));
            
    	}
    	catch(error)
        {
            console.error(error.message);
        }
	}
	
	getDaydegrees (data, start, end, name) {
		
		let modelConfig = {
			"loginInfo":{
				"username":"testuser",
				"password":"testpass"
			},
			"modelId":"DAYDEGREES",
			"configParameters":{
				"basisTemp": this.basisTemp,
				"observations": data
			}
		};
	
   	 	let request = $.ajax({
        	type:"POST",
        	url: this.runUri,
        	data: JSON.stringify(modelConfig),
        	dataType: "json",
        	contentType: "application/json; charset=utf-8",
    	})
    	.done((data, textStatus, jqXHR) => {
			this.displayResults(data, start, end, name);
		})
		.fail((jqXHR, textStatus,errorThrown ) => {
			alert( "Request failed: " + errorThrown );
		})
		;
	}
	
	
	setStartText () {
		
		document.getElementById("textInfo").innerText = "";		  
			
		let infotext = document.createElement("p");
		infotext = "<p>Varmesum, med basistemperatur 0 °C, vert rekna ut frå middeltemperaturen 2 m over bakken for den valde vêrstasjonen. </p>";		  	
			
		document.getElementById("textInfo").innerHTML=infotext;
	}



	
	displayResults (data, start, end, name) {
	
			let date = new Array();
			let series = [];
			let time = new Array();
			let temperature = new Array();
			let forecast = new Array();
			
			let emptyChart = false;
				
			if(!emptyChart) {
				for (let i = 0; i < data.length; i++){
					
		        	let localTime = new Date(data[i].validTimeStart);
		        	const offset = localTime.getTimezoneOffset()
					localTime= new Date(localTime.getTime() - (offset*60*1000))
						
					if(localTime < moment()){
						temperature.push(parseInt((data[i].allValues).match(/[-+]?([0-9]*\.[0-9]+|[0-9]+)/)));
						forecast.push(null);
					} else{
						forecast.push(parseInt((data[i].allValues).match(/[-+]?([0-9]*\.[0-9]+|[0-9]+)/)));
					}
				
		    	    localTime = localTime.toISOString().split('T')[0];
		        	time.push(localTime);
				}
				
				// Fill the gap between temperature and forecast
				let index = temperature.length-1;
				forecast[index]=temperature[index];
			
				const lastValue = forecast[forecast.length -1];
			
				// Set x-axis interval to 7 days
    			var maxTicksLimit = time.length / 7;
    	    	var rotation = 0;
    			if(maxTicksLimit > 10) {
					rotation = 20;
					maxTicksLimit = 11;
				}
				
				document.getElementById("textInfo").innerText = "";		  
			
				let infotext = document.createElement("p");
				let text1 = "<p>Varmesum, med basistemperatur 0 °C, rekna ut frå middeltemperaturen 2 m over bakken for den valde vêrstasjonen. </p>";
				let text2 = "<p><b>" + lastValue + "</b> døgngrader\n\n</p>";	
						  	
	    		infotext = text1 + text2;
		
				document.getElementById("textInfo").innerHTML=infotext;
				
			} else{
				for (var i = 0; i < 10; i++){
					time.push(null);
					forecast.push(null);
					name = '-';
					start = '-';
					end = '-'; 
				}
			}
			
			let chartExist = Chart.getChart("daydegreesum");
			
			if(chartExist != undefined){
				chartExist.destroy();
			}
			
			let ctx = document.getElementById("daydegreesum").getContext('2d');
			
			series[0] = {
					label: "Døgngrader",
					data: temperature,
					borderColor: '#ff0000',
					borderWidth: 2,
					spanGaps: true,
					fill: {
						target: 174,
						above: 'rgb(255, 0, 0)',   // Area will be red above the origin
                		below: 'rgb(0, 0, 255)'    // And blue below the origin
                	}
			};
			
			series[1] = {
					label: "Døgngrader prognose",
					data: forecast,
					borderColor: '#ff0000',
					borderDash: [3, 3],
					borderWidth: 2,
					spanGaps: true
			};
					
			let dataset = {
					labels: time,
					datasets: series
			};
				
			let text = String("Varmesum for " + name + " i perioden " + start + " til " + end );
			
			new Chart(ctx, {
				type: 'line',
			    data: dataset,
			    options: {
			    	scales: {
						x: {
							ticks: {
								maxTicksLimit: maxTicksLimit,
								maxRotation: rotation
							}
						},
		    			y: {
		    				display: true,
		    				title: {
		    					display: true,
		    					text: '°C'
		    				}
		    			},
		    		},
		    		elements: {
		    			point:{
		    				radius: 0
		    			}
		    		},	    		
		    		plugins: {
		    			title: {
		    				display: true,
		    				text: text,
		    				fontSize: 18
		    			},
		    			legend: {
		    				display: true,
		    				labels: {
		    					boxHeight: 1
		    				},
		    				position: 'bottom'
		    			},
		    			tooltip: {
							intersect: false,
							displayColors: false,
		    				callbacks: {
								title: function () {
									return '';
								},
								label: function (context) {
									const date = context.label;
									const value = context.parsed.y;
									return `${date} er varmesummen ${value} °C`;
								}
							}
		    			 },
		    			 annotation: {
							annotations: [{
								type: 'line',
								mode: 'horizontal',
								scaleID: 'y',
								value: 174,
								borderColor: 'grey',
								borderWidth: 2,
								borderDash: [5, 5],
								label: {
          							color: 'grey',
          							backgroundColor: 'transparent',
          							content: 'Sverming',
          							font: {
            							size: 14
            						},
            						enabled: true,
            						position: 'end',
            						xAdjust: 10,
            						yAdjust: -10
            					}
							}, {
								type: 'line',
								mode: 'horizontal',
								scaleID: 'y',
								value: 320,
								borderColor: 'grey',
								borderWidth: 2,
								borderDash: [5, 5],
								label: {
          							color: 'grey',
          							backgroundColor: 'transparent',
          							content: 'Egglegging',
          							font: {
            							size: 14
            						},
            						enabled: true,
            						position: 'end',
            						xAdjust: 10,
            						yAdjust: -10
            					}
							}, {
								type: 'line',
								mode: 'horizontal',
								scaleID: 'y',
								value: 565,
								borderColor: 'grey',
								borderWidth: 2,
								borderDash: [5, 5],
								label: {
          							color: 'grey',
          							backgroundColor: 'transparent',
          							content: 'Klekking',
          							font: {
            							size: 14
            						},
            						enabled: true,
            						position: 'end',
            						xAdjust: 10,
            						yAdjust: -10
            					}
							}
							]
						}
					}
				}
			  });
			 
	}
	
}

export default ApplefruitMoth;
